(function ($, Drupal, once) {

  "use strict";

  /**
   * Adds event listeners to elements within a menu tree structure.
   *
   * @returns {void}
   */
  const addEventListeners = () => {
    // Add click event listener to BUTTON and LI elements.
    const triggers = Array.from(document.querySelectorAll('.menu-tree button, .menu-tree li'));
    window.addEventListener('click', e => {
      let elem = e.target;
      if (triggers.includes(elem)) {
        if (elem.tagName === 'BUTTON') {
          e.preventDefault();
        }
        else {
          elem = elem.querySelector('button:not([disabled])');
        }
        triggers.forEach(el => {
          el.classList.remove('selected');
          el.parentElement.classList.remove('selected');
        });
        elem.classList.add('selected');
        elem.parentElement.classList.add('selected');
        setState(elem.getAttribute('data-value'));
        addMenuLinkAsChild(elem);
        removeEmptyLists();
      }
    }, false);

    // Add dragstart/dragend event listeners to existing menu item.
    const menuItem = document.querySelector('.menu-tree li[draggable="true"]');
    if (menuItem) {
      createDraggableMenuLink('', menuItem);
    }
    else {
      setInitialState();
    }

    // Add dragover event listeners to drop targets.
    const dropTargets = document.querySelectorAll('ul.menu-tree ul');
    dropTargets.forEach(container => {
      container.addEventListener('dragover', function(e) {
        const draggedElement = document.querySelector(".menu-tree li.dragging");
        const afterElement = getDragAfterElement(container, e.clientY);
        if (afterElement === undefined) {
          this.appendChild(draggedElement);
        }
        else {
          afterElement.parentElement.insertBefore(draggedElement, afterElement);
        }
      })
    });

    // Add an event listener to the custom jQuery event 'formUpdated' to exactly mirror the value
    // of the menu link title input to the draggable menu link. This is the only place where jQuery
    // is used in this script, but it seems to be the only solution.
    const input = document.querySelector('input[name="menu[title]"]');
    $(input).on('formUpdated', (e) => {
      input.dispatchEvent(new Event('input'));
    });

    // Add an event listener on the "Provide a menu link" checkbox. We add the event listener to
    // the window object to make sure it runs after 'menu_ui' has done its menu link title value
    // mirroring from the node title field.
    window.addEventListener('change', (e) => {
      if (e.target.matches('input[name="menu[enabled]"]')) {
        input.dispatchEvent(new Event('input'));
      }
    });

    // Add an event listener on the menu title input.
    input.addEventListener('input', e => {
      const elem = document.querySelector('.menu-tree li[draggable="true"]');
      if (elem) {
        elem.querySelector('button').innerHTML = e.target.value;
      }
      else {
        createDraggableMenuLink(e.target.value);
      }
    });
  }

  /**
   * Add the new menu item as a child of the selected menu item.
   *
   * @param {HTMLElement} selected - The selected menu parent.
   * @returns {void}
   */
  const addMenuLinkAsChild = (selected) => {
    const elem = document.querySelector('.menu-tree li[draggable="true"]');
    if (!elem) return;
    if (selected.parentElement.tagName === 'SUMMARY') {
      const ul = selected.parentElement.nextElementSibling;
      ul.appendChild(elem);
    }
    else {
      selected.classList.add('selected');
      const details = document.createElement('details');
      details.setAttribute('open', 'true');
      selected.parentElement.appendChild(details);
      const summary = document.createElement('summary');
      summary.classList.add('selected');
      summary.setAttribute('aria-expanded', 'true');
      summary.appendChild(selected);
      const ul = document.createElement('ul');
      ul.classList.add('branch');
      ul.appendChild(elem);
      details.appendChild(summary);
      details.appendChild(ul);
    }
  }

  /**
   * Initializes the menu state by selecting the appropriate elements based on
   * the value of the 'menu[menu_parent]' input element.
   *
   * @returns {void}
   */
  const setInitialState = () => {
    const menuParent = document.querySelector('input[name="menu[menu_parent]"]');
    setState(menuParent.value);
  }

  /**
   * Set the state of the menu based on the currently selected parent element.
   *
   * @param {string} value - The ID of the selected parent link.
   * @returns {void}
   */
  const setState = (value) => {
    // Remove the class 'selected' on all elements.
    const previous = document.querySelectorAll('.menu-tree .selected');
    previous.forEach(el => {
      el.classList.remove('selected');
    });

    // Process the selected element.
    let elem = document.querySelector('.menu-tree button[data-value="' + value + '"]');
    if (elem) {
      elem.classList.add('selected'); // BUTTON element.
      elem.parentElement.classList.add('selected'); // SUMMARY element.

      // Set the menu parent input value.
      const menuParent = document.querySelector('input[name="menu[menu_parent]"]');
      menuParent.value = value;

      // Open all parent DETAILS elements.
      while (elem) {
        if (elem.matches('details')) {
          elem.setAttribute('open', 'open');
        }
        elem = elem.parentElement;
      }

      // Save references to the sibling elements in the form.
      let menuItem = document.querySelector('.menu-tree li[draggable="true"]');
      while (menuItem) {
        if (menuItem.tagName === 'LI') {
          // Get the previous sibling element.
          const prevInput = document.querySelector('input[name="menu[prev_sibling]"]');
          if (
            menuItem.previousElementSibling
            && menuItem.previousElementSibling.matches('li')
          ) {
            prevInput.value = menuItem.previousElementSibling.querySelector('button').getAttribute('data-value');
          }
          else {
            prevInput.value = '';
          }

          // Get the next sibling element.
          const nextInput = document.querySelector('input[name="menu[next_sibling]"]');
          if (
            menuItem.nextElementSibling
            && menuItem.nextElementSibling.matches('li')
          ) {
            nextInput.value = menuItem.nextElementSibling.querySelector('button').getAttribute('data-value');
          }
          else {
            nextInput.value = '';
          }

          break;
        }
        menuItem = menuItem.parentElement;
      }
    }
  }

  /**
   * Creates a draggable menu link item and appends it to the selected menu tree.
   *
   * @param {string} [menuTitle] - The title displayed for the newly created menu button.
   * @param {HTMLElement} [elem] - The draggable menu link element.
   * @returns {void}
   */
  const createDraggableMenuLink = (menuTitle, elem) => {
    // Create the element if it does not exist.
    if (typeof elem === 'undefined') {
      console.log('undefined element');
      const selected = document.querySelector('.menu-tree button.selected');
      elem = document.createElement('li');
      elem.setAttribute('draggable', 'true');
      elem.innerHTML = '<button disabled data-value="" data-weight="">' + menuTitle + '</button>';
      document.querySelector('.menu-tree ul.root').appendChild(elem);
      addMenuLinkAsChild(selected);
    }

    // Set the current state.
    setState(elem.parentElement.previousElementSibling.querySelector('button').getAttribute('data-value'));

    // Add event listeners.
    elem.addEventListener('dragstart', function() {
      this.classList.add('dragging');
    });
    elem.addEventListener('dragend', function() {
      this.classList.remove('dragging');
      // Update state efter menu link has been placed.
      setState(this.parentElement.previousElementSibling.querySelector('button').getAttribute('data-value'));
      removeEmptyLists();
    });
  }

  /**
   * Remove empty UL elements and parent DETAILS element to prevent an item
   * rendering as having children.
   *
   * @returns {void}
   */
  const removeEmptyLists = () => {
    const ulElems = document.querySelectorAll('.menu-tree ul.branch');
    const emptyElems = Array.from(ulElems).filter(elem => elem.childElementCount === 0);
    emptyElems.forEach(elem => {
      const newParent = elem.parentElement.parentElement; // UL element.
      const currParent = elem.parentElement; // DETAILS element.
      const button = currParent.querySelector('button');
      newParent.appendChild(button);
      currParent.remove();
    });
  }

  /**
   * Determines the element after which a dragged item should be placed based on a given vertical position.
   *
   * @param {HTMLElement} container - The parent container element that holds the list items.
   * @param {number} y - The vertical position (usually the mouse or pointer position) used to determine placement.
   * @returns {HTMLElement|null} The element after which the dragged item should be inserted, or null if none is found.
   */
  const getDragAfterElement = (container, y) => {
    const notDraggedItems = [...container.querySelectorAll('.menu-tree ul li:not(.dragging)')]

    return notDraggedItems.reduce((closest, child) => {
      const box = child.getBoundingClientRect();
      const offset = y - box.top - box.height / 2;
      if (offset < 0 && offset > closest.offset) {
        return { offset, element: child }
      } else return closest;
    }, { offset: Number.NEGATIVE_INFINITY }).element;
  }

  /**
   * Attach event listeners and set the initial state of the menu.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.menuTree = {
    attach: function (context) {
      once('menu-tree', '.menu-tree-wrapper', context).forEach(elm => {
        addEventListeners();
      });
    }
  };

})(jQuery, Drupal, once);
